#
# This EEM policy detects full interface input queues. When a full queue
# has been detected, a syslog message is sent. The number of items in each 
# interface input queue, as well as the maximum queue length, is determined 
# using the EXEC command 'show interfaces'.
#
# The following is an example of the syslog message produced by this policy.
#
# %HA_EM-4-LOG: tmpsys:/eem_policy/interface-input-q.tcl: Interface 
# Ethernet0/0 input queue full. Input queue: 76/75 (size/max)
#
# v20080304
#

#
# Register this policy as time based using a watchdog timer. It will be 
# triggered every 60 seconds.
#
::cisco::eem::event_register_timer watchdog name sixtySeconds time 60

# Import the cisco namespaces that allows access to CLI procedures.
namespace import ::cisco::eem::*
        
#
# The environment variable EEM_INTERFACE_INPUT_Q_DEBUG must exist for this 
# script to run. If this variable is set to 'yes', the queue depth of all 
# interfaces is reported via syslog. Otherwise, syslog messages will only 
# be sent for queues where the queue depth exceeds the configured maximum.
#
if {![info exists EEM_INTERFACE_INPUT_Q_DEBUG]} {
 set result \
 "Policy cannot be run: variable EEM_INTERFACE_INPUT_Q_DEBUG has not been set"
 error $result $errorInfo
}
        
#
# Obtain the output of the 'show interfaces' command and place it into the 
# variable cmd_output.
#
if [catch {cli_open} result] {
 error $result $errorInfo
} else {
 array set cli $result
}
if [catch {cli_exec $cli(fd) "show interfaces"} result] {
 error $result $errorInfo
} else {
 set cmd_output $result
}
if [catch {cli_close $cli(fd) $cli(tty_id)} result] {
 error $result $errorInfo
}

#
# Parse the command output using a regular expression and place the results 
# in the list variable named list.
#
# Subinterfaces will be skipped by matching only interface names that do not 
# contain a '.'.
#
catch { \
 regexp -all -inline -lineanchor \
 {^([^\.\s]+?) is .*?, line protocol is (\S+).*?^\s*Input queue: (\d+)/(\d+)/} \
 $cmd_output
} list
        
#
# After cmd_output has been parsed by the regular expression, the variable 
# list contains a set of data for each interface. Each set contains five 
# items, four of which were specifically stored using ()s within the regexp 
# command. The five items are:
#
#  - the entire matched string
#  - interface name
#  - interface line protocol
#  - input queue size
#  - input queue max
#
set list_index 0
set stored_items 4 
        
#
# Perform the following code for each interface in the list variable list.
#
while {$list_index < [llength $list]} {

 # Copy the data from the list to more readable variable names
 set interface_name          [lindex $list [expr $list_index + 1]]
 set line_protocol_state     [lindex $list [expr $list_index + 2]]
 set input_queue_size        [lindex $list [expr $list_index + 3]]
 set input_queue_max         [lindex $list [expr $list_index + 4]]
 
 #
 # Send a syslog message if an interface is up, and its queue depth exceeds 
 # the input queue size.
 # 
 if { $line_protocol_state == "up" 
      && ($input_queue_size >= $input_queue_max) } {

  action_syslog priority warning msg \
   "Interface $interface_name input queue full.\
   Input queue: $input_queue_size/$input_queue_max (size/max)"

 } else {
  if { [string compare -nocase $EEM_INTERFACE_INPUT_Q_DEBUG "yes"] == 0 } { 

   #
   # If debugging has been enabled via the environment variable, send a syslog 
   # for each interface regardless of queue depth.
   #
   action_syslog priority debug msg  \
    "Interface $interface_name input queue at $input_queue_size of $input_queue_max"
        
  }
 }
        
 # Advance the list_index to the next set of items in the list
 incr list_index [expr $stored_items + 1]
}
